﻿/*! \file KineticScroller.as
 *
 * Implementation of kinetic scrolling with ActionScript.
 * A class, can be extended?
 *
 * Written by Antti-Jussi Kovalainen
 */

class fi.ajk.KineticScroller extends MovieClip {
	// private:

	private var state : Number; 		// 0 = idle, 1 = hover, 2 = grabbed, 3 = gliding
	private var mc : MovieClip;			// this movieclip
	private var container : MovieClip; 	// reference to container movieclip
	private var bounds : Object;

	private var dragstart : Array;
	private var delta : Array;
	private var speed : Array;

	private var target : Array;
	private var last_pos : Array;
	
	private var start_time : Number;
	private var delta_pos : Array;
	
	private var bounce_speed : Number = 2;
 	private var deaccel : Number = 10;
	private var accel : Number = 100;
	private var accel_magic : Number = 500;

	// Check if the user panned over the boundaries and bounce back
	private function checkBounds() {
		if( this._y > this.bounds.top ){
			this.target[1] = this.bounds.top;
			this.last_pos[1] = this._y;
			this.onEnterFrame = this.bounceBack;
		}
		else if( this._y < this.bounds.bottom ){
			this.target[1] = this.bounds.bottom;
			this.last_pos[1] = this._y;
			this.onEnterFrame = this.bounceBack;
		}
		else {
			// Not out of bounds, let it glide
			this.onEnterFrame = this.glide;
		}
	}

	// bounce back to bounds
	private function bounceBack() {
		if( this._y != this.target[1] ){
			this._y = this.target[1] - ( this.target[1] - this._y ) / this.bounce_speed;

			if( this._y == this.last_pos[1] ){
				this._y = this.target[1];
				//this.removeMovieClip();
			}
			this.last_pos[1] = this._y;
		}
		else {
			if( this.state == 3 ){
				this.state = 0;
			}
			delete this.onEnterFrame;
		}
	}
	
	// Glides the element after the use has released his finger
	private function glide() {
		// stop gliding, so we won't go on forever
		if( Math.abs(this.speed[1]) < 1 ){
			this.state = 0;
			delete this.onEnterFrame;
			return;
		}
		
		// check if we've glided over the bounds
		if( this._y >= this.bounds.top ){
			this._y = this.bounds.top;
			this.state = 0;
			delete this.onEnterFrame;
			return;
		}
		if( this._y <= this.bounds.bottom ){
			this._y = this.bounds.bottom;
			this.state = 0;
			delete this.onEnterFrame;
			return;
		}
		
		// slow down
		if( this.speed[1] > 0 ){
			this.speed[1] -= this.speed[1] / this.deaccel;
		} 
		else if( this.speed[1] < 0 ){
			this.speed[1] += -this.speed[1] / this.deaccel;
		}
		this._y += speed[1];

		// only go at set max speed
/*		if( vspeed > Math.abs( this.vector_max[1] ) ){
			this._y -= this.vector_max[1];
		}
		else {
			// otherwise, ease out!
			this._y -= Math.round( this.vector[1] );
		}*/
	}

	// Move the element and update speed, called when user moves finger
	private function moving() {
		if( this.state == 2 ){
			this._y = (_root._ymouse - this.delta[1]);
			delta_pos = [_root._xmouse - dragstart[0], _root._ymouse - dragstart[1]];
			speed = [accel * delta_pos[0] / accel_magic, accel * delta_pos[1] / accel_magic];
		}
	}
	
	// 
	private function letGo() {
		if( this.state == 2 ){ // if grabbing
			this.state = 3;

			this.delta_pos = [_root._xmouse - dragstart[0], _root._ymouse - dragstart[1]];
			var dt : Number = (new Date()).getTime() - this.start_time;
			
			this.speed = [accel * delta_pos[0] / dt, accel * delta_pos[1] / dt];

			this.checkBounds();
		}
	}
	
	// 
	private function pressed() {
		if (this.state == 3) {
			delete this.onEnterFrame;
		}
		
		this.state = 2;

		this.dragstart = [_root._xmouse, _root._ymouse];
		this.delta = [(_root._xmouse - this._x), (_root._ymouse - this._y)];
		
		this.start_time = (new Date()).getTime();
		this.speed = [0, 0];
	}
	
	
	// public:
	
	/**
	 * Constructor
	 *
	 */
	public function KineticScroller() {
		this.mc = this;
		this.state = 0;
		
		this.start_time = null;
		this.delta = [null, null];

		this.target = [null, null];
		this.last_pos = [null, null];
		
		this.onPress = pressed;
		this.onMouseMove = moving;
		this.onMouseUp = letGo;
		
	}
	
	public function init( movieclip : MovieClip ) {
		this.container = movieclip;
		
		this.mc._x = this.container._x;
		this.mc._y = this.container._y;
		
		this.bounds = {
			left: container._x,
			right: (container._y + this._width),
			bottom: (container._height - this._height),
			top: container._y
		}
	}	

}
